/**
**/
#include <irrlicht.h>
#include "driverChoice.h"
#include "resolutionChoice.h"
#include <windows.h>
#include <iostream>


using namespace irr;


using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;


#ifdef _IRR_WINDOWS_
#pragma comment(lib, "Irrlicht.lib")
#pragma comment(linker, "/subsystem:windows /ENTRY:mainCRTStartup")
#endif



//core::dimension2d<u32> windowSize(640, 480);

class MyProgram : public IEventReceiver
{
public:

    video::IRenderWindow* Window1;
    ILogger* Logger ;
    video::IVideoContext *Context1;
//    , *Context2;
    IrrlichtDevice *Device;
    video::IVideoDriver* VideoDriver;
    gui::IGUIEnvironment* guienv;
    scene::ISceneManager *smgr;
    video::SRenderWindowConfig WindowConfig;
    video::SVideoContextConfig ContextConfig;

    MyProgram() :
        Logger(0)
        , Window1(0)
        , Context1(0)
        , Device(0)
        , VideoDriver(0)
        , guienv(0)
        , smgr(0)
    {
        //TEST
        WindowConfig.Fullscreen = false;
        WindowConfig.Vsync = true;
        WindowConfig.Resizable = false;

    };


    bool OnEvent(const SEvent& event)
    {

        switch (event.EventType)
        {
        case EET_KEY_INPUT_EVENT:
            if(event.KeyInput.PressedDown)
                return false;

            switch(event.KeyInput.Key)
            {
                case irr::KEY_KEY_A:


                return true;

            case irr::KEY_KEY_Z:

                break;

            case KEY_KEY_E:
                Logger->log("Switching cursor visibility",ELL_INFORMATION);
                {
                     gui::ICursorControl* cursor = Device->getCursorControl();

//                    if( Device->getCursorControl()->isVisible() )
//                    {
                        //!
                        cursor->setVisible( !cursor->isVisible() );
//                    }
                    //cursor->isVisible()
                    Logger->log("Cursor visibility", ELL_INFORMATION);
                }
                break;

            // switch betwenn resizable or not
            case KEY_KEY_R:
                Logger->log("Switching between resizable window",ELL_INFORMATION);
                {
                    bool resizable = Window1->isResizable();

                    if(!Window1->setResizable( ! resizable ))
                    {
                        //!
                        Logger->log("Could not change resizable modes",ELL_WARNING);
                    }
                }
                break;
            case KEY_KEY_T:
                Logger->log("Switching between fullscreen modes",ELL_INFORMATION);
//                Logger->log("Switching between resizable window",ELL_INFORMATION);
                {
                    bool fullscreen = Window1->isFullscreen();

                    if(!Window1->setFullscreen( ! fullscreen ))
                    {
                        //!
                        Logger->log("Could not change resizable modes",ELL_WARNING);
                    }
                }
                break;

            case KEY_KEY_Q:
                Logger->log("Resize to 300x300",ELL_INFORMATION);
                if(!Window1->resize( core::dimension2du(300,300) ))
                {
                    //!
                    Logger->log("Could not resize window",ELL_WARNING);
                }
                break;

            case KEY_KEY_S:
                Logger->log("Resize to 600x600",ELL_INFORMATION);
                if(!Window1->resize( core::dimension2du(600,600) ))
                {
                    //!
                    Logger->log("Could not resize window",ELL_WARNING);
                }
                break;
            default:
                break;
            }
            break;

            case EET_WINDOW_EVENT:

                if(event.WindowEvent.RenderWindow == Window1)
                {

                    Logger->log("window 1 killed ", ELL_INFORMATION);
                    Window1 = 0;
                }

                return true;

            default:
                break;
        }





        return false;
    }


    bool renderSceneForContext( IVideoContext* context , IRenderWindow* window, const core::vector3df& camPos)
    {

        // 1st pass
        if(!VideoDriver->changeRenderContext( context, window) )
        {
            Logger->log("Could not change render context");
            return false;
        }


        if( !VideoDriver->beginScene(true, true, SColor(255,100,101,140) ) )
        {
            Logger->log("beginScene failed", ELL_ERROR);
        }


        Device->getSceneManager()->getActiveCamera()->setPosition(camPos);
        Device->getSceneManager()->drawAll();
        Device->getGUIEnvironment()->drawAll();


        if( !VideoDriver->endScene() )
        {
            Logger->log("endScene failed", ELL_ERROR);
        }

    }

    bool run()
    {
//        while(device->run())
//        {
//
//        if( program.Window1 )
//        {
//            // 1st pass
//            if(!driver->changeRenderContext( context1, Window1) )
//            {
//                logger->log("Could not change render context");
//                return 0;
//            }
//

        return false;
    }


    //!
    bool init(IrrlichtDevice *device)
    {
        if( device == 0 )
            return false;


        Device = device;
        Logger = Device->getLogger();
        VideoDriver = Device->getVideoDriver();

        ////////////////////////////////////////////////////////
        ////    Create windows
        ////////////////////////////////////////////////////////
//        video::SRenderWindowConfig windowConfig;
//        windowConfig.WindowSize = windowSize;
//        windowConfig.Fullscreen = false;
//        windowConfig.Vsync = true;

        //!
        Window1 = device->createWindow(WindowConfig);
//        Window2 = device->createWindow(WindowConfig);

        if(! (Window1) )
        {
            Logger->log("Could not create window", ELL_ERROR);
            return false;
        }

        // create cursor control
        device->setActiveWindow( Window1 );


        Window1->setWindowCaption(L"First window");


        ////////////////////////////////////////////////////////
        ////    Create & set contexts
        ////////////////////////////////////////////////////////
        video::SVideoContextConfig contextConfig;
        contextConfig.ContextSize = WindowConfig.WindowSize;
        contextConfig.Doublebuffer = true;

        if ( ! VideoDriver)
        {
            Logger->log("Could not retrieve video driver", ELL_ERROR);
            return false;
        }



        Context1 = VideoDriver->createContext( contextConfig, Window1 );
        if(! Context1 )
        {
            Logger->log("Could not create first context", ELL_ERROR);
            return false;
        }



        smgr = Device->getSceneManager();
        guienv = Device->getGUIEnvironment();
        return true;
    }
};



//bool registerDriverFromDll(const video::TDriverType& id, const core::stringc& dllPath)
//{
//    //! TODO LoadLibrary does it need to be unload ?
//	HINSTANCE DriverLibrary = LoadLibrary(
////                                    __TEXT("d3d9.dll")
//
//                                    dllPath.c_str()
//                                    );
//
//		if (!DriverLibrary)
//		{
//			os::Printer::log("Error, could not load d3d9.dll.", ELL_ERROR);
//			return false;
//		}
//
//
////		typedef IDirect3D9 * (__stdcall *D3DCREATETYPE)(UINT);
//		video::TVideoDriverCreationFunction driverCreationFunction = (D3DCREATETYPE) GetProcAddress(DriverLibrary, "createDriver");
//
//
//    return registerDriver(id, driverCreationFunction);
//}


/*
This is the main method. We can now use main() on every platform.
*/
int main()
{

	// ask user for driver
//	video::E_DRIVER_TYPE driverType = driverChoiceConsole();
	video::IVideoDriver* driver = 0;

//	video::TVideoDriver driverId  ;
//	video::E_DRIVER_TYPE driverType = EDT_OPENGL;
//                 video::EDT_SOFTWARE
//            video::EDT_DIRECT3D9
//            video::EDT_BURNINGSVIDEO
            ;
//	if (driverType==video::EDT_COUNT)
//		return 1;

    //
//    const
    TDriverType driverId("d3d9_driver_debug");
    driverId = "opengl_driver_debug";
//    driverId = "opengl_driver";

	/*
    createDeviceLight
    */
    SIrrlichtCreationParameters params;
//    params.DriverType = driverType;
//    params.DriverType = driverId ;


//    params.WindowSize = windowSize;

//    params.Bits = 16;
//    params.Fullscreen = false;
//    params.Stencilbuffer = false;
//    params.Vsync = true;
//    params.EventReceiver = res;

	IrrlichtDevice *device = createDeviceLite(params);

	if (!device)
    {
//        cout << "Could not create device" << endl;
		return 1;
    }

    ILogger* Logger = device->getLogger();
    //! Load library

    core::stringc libraryName;
    libraryName.append(driverId);
    libraryName.append(".dll");
    //! TODO LoadLibrary does it need to be unload ?
	HINSTANCE DriverLibrary = LoadLibrary(
//                                    __TEXT("d3d9.dll")
//                                    __TEXT("opengl_driver_debug.dll")
                                    __TEXT( libraryName.c_str() )

//                                    dllPath.c_str()
                                    );

    if (!DriverLibrary)
    {
//			program.Logger->log(
        Logger->log("Error, could not load opengl_driver_debug.dll.", ELL_ERROR);
        return false;
    }


//		typedef IDirect3D9 * (__stdcall *D3DCREATETYPE)(UINT);
    video::TVideoDriverCreationFunction driverCreationFunction = (video::TVideoDriverCreationFunction) GetProcAddress(DriverLibrary, "createDriver");


//    driverId = chooseAmongRegisteredDrivers(device);


    if(!device->registerDriver( driverId , driverCreationFunction))
    {
        //!
        Logger->log("Error, could not register driver.", ELL_ERROR);
        return 2;
    }


    driverId = chooseAmongRegisteredDrivers(device);
    Logger->log("You 've chosend driver", driverId.c_str(), ELL_INFORMATION);


    //params.DriverType
	driver = device->loadVideoDriver( driverId );

    if(!driver)
    {
        Logger->log("Could not retrieve/load video driver", ELL_ERROR);
        return 3;
    }

//    core::dimension2du windowSize = resolutionChoiceConsole( device );
    core::dimension2du windowSize(640,480);

    MyProgram program;

	ISceneManager* smgr = 0;
	IGUIEnvironment* guienv = 0;
	ILogger* logger = device->getLogger();

	device->setEventReceiver( &program );
    logger->setLogLevel( ELL_DEBUG );


    program.WindowConfig.WindowSize = windowSize;

    if(! program.init( device ))
        return 1;


    driver = device->getVideoDriver() ;






    // 1st pass
    if( !driver->changeRenderContext( program.Context1, program.Window1) )
    {
        logger->log("Could not change render context",ELL_ERROR);
        return 1;
    }


    // IMPORTANT need a valid context (since it will load the built-in font
    // create gui and menu !
    device->createGUIAndScene();
    smgr = device->getSceneManager();
    guienv = device->getGUIEnvironment();


	IAnimatedMesh* mesh = smgr->getMesh("../../media/sydney.md2");
	if (!mesh)
	{
		device->drop();
		return 1;
	}
	IAnimatedMeshSceneNode* node = smgr->addAnimatedMeshSceneNode( mesh );

	if (node)
	{
		node->setMaterialFlag(EMF_LIGHTING, false);
		node->setMD2Animation(scene::EMAT_STAND);
		node->setMaterialTexture( 0, driver->getTexture("../../media/sydney.bmp") );
	}

	/*
	To look at the mesh, we place a camera into 3d space at the position
	(0, 30, -40). The camera looks from there to (0,5,0), which is
	approximately the place where our md2 model is.
	*/
	vector3df camPos1(0,30,-40);
	vector3df camPos2(0,-0,+40);
	smgr->addCameraSceneNode(0, camPos1, vector3df(0,5,0));

    core::rect< s32 > rectangle(0,0,400,200);
    guienv->addStaticText( L"TEST TEXT", rectangle );
// addStaticText (const wchar_t *text, const core::rect< s32 > &rectangle, bool border=false, bool wordWrap=true, IGUIElement *parent=0, s32 id=-1, bool fillBackground=false)=0



    program.run();

	while(device->run())
	{
        program.renderSceneForContext( program.Context1 , program.Window1, camPos1 );
	}


	device->drop();

	return 0;
}
