
#ifndef __IRR_I_RENDERWINDOW_H__
#define __IRR_I_RENDERWINDOW_H__

#include "IrrCompileConfig.h"
#include "SIrrCreationParameters.h"
#include "SColor.h"
#include "IImage.h"
#include "IRenderTarget.h"
#include "IReferenceCounted.h"
#include "SExposedVideoData.h"


namespace irr
{

namespace video
{

    class IVideoContext;

// TODO compare with SFML
enum E_RENDER_WINDOW_STYLE
{
//    ERWS_RESIZABLE,
//    ERWS_SHOW_MINIMIZE,
};


// TODO struct SRenderWindowStyle

struct SRenderWindowConfig
{

    core::dimension2d< u32 > WindowSize;
    bool Fullscreen;
    bool Vsync; //    u32 Bits;   //! should be acknowledged by driver ?
    bool Resizable;
    //u32 Bits;   //! should be acknowledged by driver ?

    SRenderWindowConfig() :
        WindowSize(core::dimension2d<u32>(800, 600)),
        Fullscreen(false)
        , Vsync(false)
        , Resizable(false)
        {
            //!
        };



};




/**
TODO remove the ID thing
**/
    class IRRLICHT_API IRenderWindow
//    : public IRenderTarget
    {
        s32 ID;

    public:
//        const SIrrlichtCreationParameters& CreationParams;

        /** Default constructor.
        const SIrrlichtCreationParameters& params
        */
//        IRenderWindow(const SRenderWindowConfig& config) :
//            ID(-1)
////        :
////            CreationParams(params)
//        {
//            //!
//        }

        //!const bool& external
        IRenderWindow(bool external) :
            ID(-1)
            , Context(0)
            , ExternalWindow(external)
        {

        }

		//! Get the id of the scene node.
		/** This id can be used to identify the node.
		\return The id. */
		virtual s32 getID() const
		{
			return ID;
		}


		//! Sets the id of the scene node.
		/** This id can be used to identify the node.
		\param id The new id. */
		virtual void setID(s32 id)
		{
			ID = id;
		}


		//! sets the caption of the window
		virtual void setWindowCaption(const wchar_t* text) = 0;

        // setPostion/getPosition are apparently available in irrlicht tracker as a patch

		//! returns if window is active. if not, nothing need to be drawn
		virtual bool isWindowActive() const = 0;


        //! AdjustWindowRect
        virtual core::dimension2d<u32> getClientSize() const = 0;

        //! relative to upper-left point of screen (0/0)
        virtual core::dimension2d<u32> getPosition() const = 0;

		virtual bool isFullscreen() const = 0;

		//! returns if window has focus
		virtual bool isWindowFocused() const = 0;

		//! returns if window is minimized
		virtual bool isWindowMinimized() const = 0;

		//! Switch to fullscreen (bool reset)
		virtual bool setFullscreen(const bool&) = 0;

		//! Notifies the device, that it has been resized TODO renommer en resize
//		void OnResized();

		//! Sets if the window should be resizable in windowed mode.
		virtual bool setResizable(bool resize=false) = 0;

		//!
		virtual bool isResizable() const = 0;


		virtual bool resize( const core::dimension2du& ) = 0;

		//! Minimizes the window.
		virtual void minimizeWindow() = 0;

		//! Maximizes the window.
		virtual void maximizeWindow() = 0;

		//! Restores the window size.
		virtual void restoreWindow() = 0;

        // bool force
        virtual void closeWindow(bool forceDestroy) = 0;

		//! Get the current color format of the window (or should it be  context ?)
		/** \return Color format of the window. */
		virtual video::ECOLOR_FORMAT getColorFormat() const = 0;


        /**

        **/
        virtual void* getWindowHandle() = 0;

//        virtual void forceDestroy() = 0;
        virtual bool attachContext(IVideoContext *context)
        {
            Context = context;
            return true;
        }

        IVideoContext* Context;
        SRenderWindowConfig Config;

        // OK
        const bool ExternalWindow;
    };


}} // Namespace

#endif

