// Copyright (C) 2002-2011 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine".
// For conditions of distribution and use, see copyright notice in irrlicht.h

#ifndef __I_VIDEO_CONTEXT_H_INCLUDED__
#define __I_VIDEO_CONTEXT_H_INCLUDED__

#include "irrTypes.h"
#include "dimension2d.h"
#include "IReferenceCounted.h"
#include "IImage.h"

namespace irr
{
namespace video
{


class IRenderWindow;

// rename into SDriverContext ?
//  SupportedConfigMax ? or combination
struct SVideoContextConfig
{
u8  AntiAlias ;
u32 Bits;
core::dimension2d<u32> ContextSize;
bool  WithAlphaChannel;
bool  Stencilbuffer ;
bool  Doublebuffer ;
bool  Stereobuffer ;
u8  ZBufferBits ;
bool HandleSRGB;
bool Vsync;
bool PureSoftware;  //!< for directX drivers
bool HighPrecisionFPU;
u8 DisplayAdapter;
ECOLOR_FORMAT ColorFormat;
bool DriverMultithreaded;

SVideoContextConfig() :
    AntiAlias(0),
    Bits(16),
    WithAlphaChannel(false),
    Doublebuffer(true),
    Stereobuffer(false),
    ZBufferBits(16),
    HandleSRGB(false),
    Vsync(false),
    PureSoftware(false),
    HighPrecisionFPU(false),
    DisplayAdapter(0),
    ColorFormat(ECF_R8G8B8),
    DriverMultithreaded(false)
    {

    }
};



class IVideoDriver;


// TODO to replace SExposedVideoData
class IVideoContext : public virtual IReferenceCounted
{
    public:

//    IVideoDriver* videoDriver,
    IVideoContext( const SVideoContextConfig& config, const bool& external) :
        Config(config)
        , ExternalContext(external)
//        , VideoDriver(0)    // TODO
    {

    }

    virtual ~IVideoContext()
    {
        //dstor
    }


    /**
    For example on windows, OpenGL rendering context does not depend on device context, so it might be displayed in several windows. NEvertheless
    PixelFormat needs to be the same
    **/
//    virtual bool isCompatibleWithRenderWindow(IRenderWindow* ) const = 0;
//    virtual bool setTarget();

    //! passer des parametres
    //bool backBuffer=true, bool zBuffer=true,SColor color=SColor(255,0,0,0),
//    virtual bool beginScene(bool backBuffer=true, bool zBuffer=true,SColor color=SColor(255,0,0,0)) = 0;
//    virtual bool endScene() = 0;

    virtual u8 getAntiAliasingLevel() const
    {
        return Config.AntiAlias;
    }

//    u8 getColorFormat() const
//    {
//        return Config.AntiAlias;
//    }

    // get current frames per second value
//    virtual s32 getFPS() const = 0;

    virtual void releaseContext() = 0;

//    bool isExternalContext() const {
//        return ExternalContext;
//    }

// setViewport

    virtual core::dimension2du getSize() const = 0;


    /**
    Might be able to change it at runtime for certain drivers (for instance by resetting a DirectX driver)
    **/
    virtual bool setVsyncMode(bool enable)
    {
        //bool enable
        return false;
    }

    //
    virtual bool setSize( const core::dimension2du& dims ) = 0;

    const SVideoContextConfig& getConfig() const
    {
        return Config;
    }


    const bool ExternalContext; //! ?

//    virtual IVideoDriver* getVideoDriver() const = 0;
//    { return VideoDriver; };
private:
//    IVideoDriver* VideoDriver;
    SVideoContextConfig Config;

};

}
}

#endif
